package contextManager;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.ObjectInputStream;

import org.akogrimo.wp42.contextmanager.entities.*;

import privacyManager.*;

public class ContextHandler {
	
	private Privacy privacy=null;
	private Context[] currentContext;
	private Position p=null;
	
	public ContextHandler(){
		
		//context owner
		User tom= new User("tom@akogrimo.org", "Tom" );
				
		//context consumers
		User alice= new User("alice@akogrimo.org", "Alice" );
		User per= new User("per@domain.no", "Per");
		
		//requested context
		Context reqContext= new Context(Context.RFID_LOCATION,""); // hva skal id vre??
		Context reqContextTwo= new Context(Context.SIP_PRESENCE,"");
		Context reqContextThree= new Context(Context.SLP_SERVICE,"");
		
		//the context owner's current context - a vector
		setContextVector(tom);
							
		//create request for alice		
		Subscription_request req= new Subscription_request(alice,tom,reqContext);
		handleAccessRights(req);
		Subscription_request reqTwo= new Subscription_request(alice,tom,reqContextTwo);
		handleAccessRights(reqTwo);	
		Subscription_request reqThree= new Subscription_request(alice,tom,reqContextThree);
		handleAccessRights(reqThree);
		
		
		//create req for per
		Subscription_request reqPer= new Subscription_request(per,tom,reqContext);
		handleAccessRights(reqPer);
		Subscription_request reqTwoPer= new Subscription_request(per,tom,reqContextTwo);
		handleAccessRights(reqTwoPer);	
		Subscription_request reqThreePer= new Subscription_request(per,tom,reqContextThree);
		handleAccessRights(reqThreePer);
		
	}
	
	public void setContextVector(User ownerUser){
		/*for testing.........*/
		int zone = 0;
		char zoneLetter = 0;
		float northing = 300000; 
		float easting = 400000; 
		float altitude = 0; 
		float accuracy = 0;
		String presenceStatus = "Occupied";
		String serviceStatus="WLAN";
				
		currentContext = new Context[3];
		currentContext[0]= new RfidContext("rfid"); 
		currentContext[1]= new SipPresenceContext("presence"); 
		currentContext[2]= new SlpService("service");
		p = new Position(zone,zoneLetter,northing,easting,altitude,accuracy);
		((RfidContext) currentContext[0]).setPosition(p);
		((SipPresenceContext) currentContext[1]).setSipContextData(presenceStatus);
		((SlpService)currentContext[2]).setContextId(serviceStatus);
		/*for testing.........*/
			
		//currentContext=ownerUser.getContext(); should only be this sentence when used in a complete system
				
	}
	
	/**
	 * This method retrives the necessary parameters from the a request object 
	 * and calls the chekcAccessRights method in the PrivacyHandler
	 * The method alsocreates the PrivacyHandler for the context owner if it does not already exist
	 * 
	 * @param  Subscriber_request
	 * @return boolean, says if access is granted
	 */
	
	public boolean handleAccessRights(Subscription_request req){
		
		String reqUserID=(req.getRequestingUser()).getId();
		int contextType= (req.getReqContextType()).getContextType();
		String ownerUserId=(req.getOwnerUser()).getId();
		boolean accessGranted=false;
				
		if(getPrivacyHandler(ownerUserId)!=null){	
		privacy= getPrivacyHandler(ownerUserId);
		privacy.setPrivacyHandler();
		System.out.println("Checks access for: "+reqUserID);
		privacy.checkAccessRights(reqUserID, contextType, currentContext);
		accessGranted=privacy.getAccessRights();
				
		Position[] pos=null;
		Position newPosMin=null; 
		Position newPosMax = null;
		
		if((privacy.getRestrictions()!=null)&&(contextType==Context.RFID_LOCATION)){
		pos= (Position [])privacy.getRestrictions();
		newPosMin = pos[0];
		newPosMax = pos[1];
		System.out.println("Access granted from contexthandler: "+accessGranted+ " Restrictions max east,north : "+newPosMax.getUtmEasting()+" , "+newPosMax.getUtmNorthing()+" Min east,north: "+newPosMin.getUtmEasting()+" , "+newPosMin.getUtmNorthing());
		}
		}
		System.out.println("Access granted from contexthandler: "+accessGranted);
		
		return accessGranted;
	}
	
	public Privacy getPrivacyHandler(String userId) {
		
		
		
		File f= new File("Policy//"+userId);
		FileInputStream in;
		try {
			in = new FileInputStream(f);
		ObjectInputStream s = new ObjectInputStream(in);
			privacy = (PrivacyHandler)s.readObject();
			s.close();
		if(in!=null)in.close();
		} catch (FileNotFoundException e) {
			System.out.println("It does not exist a privacy handler for this user.");
			privacy=null;
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
				
			return privacy;
		}
			
 
	public static void main(String[] argv) {
		new ContextHandler();
	}
	
	
}
