package privacyManager;

import java.io.*;
import java.sql.Time;
import java.util.*;
import org.akogrimo.wp42.contextmanager.entities.*;


public class PrivacyHandler implements Serializable, Privacy{

	private static final long serialVersionUID = 1L;
	private boolean accessGranted=false;
	private ArrayList listOfGroups = new ArrayList();
	private ArrayList listOfContextType = new ArrayList();
	private ArrayList listOfCurrentContext= new ArrayList();
	private String userID;
	private Matrix matrix=null;
	private Context currentLocation;
	private AccessPolicy policy=null;
	private Object[] restrictions=null;
	private Restrictions r=null;

	
	public PrivacyHandler(String userID){
		this.userID=userID;	
		setPrivacyHandler();				
		writeToFile();
	}
	
	public void setPrivacyHandler(){
		policy=getPolicy(userID);
		
		matrix=getAccessMatrix(userID);
		listOfGroups=policy.getListOfGroups();
		listOfContextType=policy.getListOfTypes();
		listOfCurrentContext=policy.getListOfCurrent();
		
	}
	
	
	public String getUserID(){
		return userID;
	}
	
	public boolean getAccessRights(){		
		
		return accessGranted;
	}
	
	/**
	 * This method checks if the context consumer is allowed to receive the requeseted information and
	 * sets the restriction and access granted parameters.
	 *  
	 * @param  requestorID, the ID of the context consumer
	 * @param  demandedContext, the context that the consumer what to receive
	 * @return void
	 */
	public void checkAccessRights(String requestorID, int demandedContext, Context[] currentContext){
		
		matrix=getAccessMatrix(userID);
		HashMap listGroup=checkIfUserInGroup(requestorID);
		HashMap listCurrent=checkCurrentContext(currentContext);
		boolean checkGroup = false;
		boolean checkCurrent = false;
		boolean checkType = false;
		
		int group=0;
		int situation=0;
		int demandedType=0;
		
		accessGranted=false;
			
			for(int i=0;i<listOfContextType.size();i++){
				SharedContext type=(SharedContext) listOfContextType.get(i);
			
				if(type.getContextType()==demandedContext){
					checkType = true;
					demandedType=i;
				}
					
				if(listGroup.containsValue(new Integer(i))==true){
						checkGroup = true;
						group=i;
				}
		
				if(listCurrent.containsValue(new Integer(i))==true){
						checkCurrent = true;
						situation=i;
					}
				
				if(checkCurrent==true&&checkGroup==true&&checkType==true){
							if(matrix.getAccessRights(group,demandedType,situation)[0]==1){
							//the accessRight is granted
							accessGranted = true;
							
							//checks for restrictions
							int contextType=((SharedContext)listOfContextType.get(demandedType)).getContextType();
							int detailLevel=matrix.getAccessRights(group,demandedType,situation)[1];
							
							if (contextType==Context.RFID_LOCATION){
								r = new DetailLevelLocation(detailLevel);
							}
							
							//here it is possible to add restrictions for other types of context..
							if(r!=null){
							r.setRestrictions(detailLevel,currentLocation);							
							restrictions= r.getRestrictions();
							}	
					}	
						
				}
			}
		}	
	
//	}
	
	public Object[] getRestrictions(){
		return restrictions;
		
	}
	
	/**
	 * Checks which groups the context consumer is a part of
	 *  
	 * @param  requestorID, the ID of the context consumer
	 * @return HashMap of all the groups the context consumer is a member of
	 */
	public HashMap checkIfUserInGroup(String requestorID){
		HashMap memberList = new HashMap();
		for(int i=0;i<policy.getListOfGroups().size();i++){
			AccessGroups aGroup=(AccessGroups)listOfGroups.get(i);
				if((aGroup.getUserList()).containsValue(requestorID)==true){
					memberList.put(aGroup, new Integer(i));
								
				}
		}//for
		
		return memberList;
		
	}
	
	/**
	 * Checks which current context situations that applies at this moment and sets the
	 * currentLocation parameter which is used to compute restrictions
	 *  
	 * @param  contextVector, a vector of all the user's current context parameters
	 * @return HashMap of all the situations that applies at this moment
	 */	
	public HashMap checkCurrentContext(Context[] contextVector){
		Context vectorContext;		
		CurrentContext current;
		int currentType = 0;
		int vectorType =0;
		HashMap listCurrent=new HashMap();
		boolean locationOk=false;
		boolean presenceOk=false;
		boolean serviceOk=false;
		
		for(int i=0;i<listOfCurrentContext.size();i++){
				current=(CurrentContext)listOfCurrentContext.get(i);
			for(int t=1;t<3; t++){
				if (current.getContextList().containsKey(new Integer(t))){
				currentType=t;
				}
				for(int k=0; k<contextVector.length;k++){
					vectorContext=(Context) contextVector[k];
					vectorType =vectorContext.getContextType();	
					
					if(vectorType==currentType){		
						
						//location values						
						if(currentType==Context.RFID_LOCATION){
							locationOk=false;
							if((checkLocationValues(current,(RfidContext) contextVector[k])==true)&&(checkTime(current)==true)){
								
								currentLocation=(RfidContext) contextVector[k];
								locationOk=true;
								
								}	
							}
							
							//presence
							if (currentType==Context.SIP_PRESENCE){		
								presenceOk=false;
								if((checkPresenceValues(current,(SipPresenceContext) contextVector[k]))==true&&(checkTime(current)==true)){
									presenceOk=true;
									
								}
							}
							
							//SLP service
							if(currentType==Context.SLP_SERVICE){
								serviceOk=false;
								if((checkServiceValues(current,(Context) contextVector[k]))==true&&(checkTime(current)==true)){
									serviceOk=true;
									
								}
							}
							
							boolean locationInList=current.getContextList().containsKey(new Integer(Context.RFID_LOCATION));
							boolean presenceInList=current.getContextList().containsKey(new Integer(Context.SIP_PRESENCE));
							boolean serviceInList=current.getContextList().containsKey(new Integer(Context.SLP_SERVICE));
													
							if((locationOk==true)||locationInList==false){
								if(presenceOk==true||presenceInList==false){
									if(serviceOk==true||serviceInList==false){
										listCurrent.put(current,new Integer(i));									}
								}
							}
						}//if
				}
			}//for		
		}//for
		
		return listCurrent;		
	}//metode
	
	/**
	 * Checks if the location values match the values of the current context situation
	 *  
	 * @param  currentContext, the users predefined situation parameters and vectorContext, the user's context
	 * 		   at this moment
	 * @return boolean valuesEqual, true if the values are equal, false if not
	 */	
	public boolean checkLocationValues(CurrentContext current, RfidContext vectorContext){
		boolean valuesEqual = false;
		int areaCorrection = 100; //correction of location - should allow access if the user is inside an area and not an exact possision
		
		float valueXmax=current.getXcoord()+areaCorrection;
		float valueYmax=current.getYcoord()+areaCorrection;
		float valueXmin=current.getXcoord()-areaCorrection;
		float valueYmin=current.getYcoord()-areaCorrection;
		float vectorValueX =vectorContext.getPosition().getUtmNorthing();
		float vectorValueY = vectorContext.getPosition().getUtmEasting();
		if ((valueXmin<vectorValueX)&&(vectorValueX<valueXmax)&&(valueYmin<vectorValueY)&&(vectorValueY<valueYmax)){
			valuesEqual = true;
		}	
		
		return valuesEqual;
	}
	
	/**
	 * Checks if the current time of the day matches the time interval of the current context situation
	 *  
	 * @param  currentContext, the users predefined situation parameters 
	 * @return boolean time, true if the values are equal, false if not
	 */		
	public boolean checkTime(CurrentContext current){
		Time start = (current.getTime())[0];
		Time stop = (current.getTime())[1];
		boolean time=false;
		Date now = new Date();
		int hours= now.getHours();
		int minutes=now.getMinutes();
		int sec=now.getSeconds();
		Time nowTime = new Time(hours,minutes,sec);
		if((start.before(nowTime)==true)&&(stop.after(nowTime)==true)){
			
			time=true;
		}
		
		return time;
	}
	
	/**
	 * Checks if the current presence value matches the value of the current context situation
	 *  
	 * @param  currentContext, the users predefined situation parameters and context which gives the user's
	 * 		   current presence context
	 * @return boolean valuesEqual, true if the values are equal, false if not
	 */		
	public boolean checkPresenceValues(CurrentContext current, Context context){
		boolean valuesEqual = false;
		
		String presenceStatus=current.getPresenceStatus();
		String vectorValue =((SipPresenceContext) context).getSipContextData();
		if (presenceStatus.equals(vectorValue)){	
			valuesEqual = true;
		}	
		return valuesEqual;
	}
	
	/**
	 * Checks if the current service value matches the value of the current context situation
	 *  
	 * @param  currentContext, the users predefined situation parameters and context which gives the user's
	 * 		   current presence context
	 * @return boolean valuesEqual, true if the values are equal, false if not
	 */			
	public boolean checkServiceValues(CurrentContext current, Context context){
		boolean valuesEqual = false;
		
		if(((SlpService)context).getContextId()==current.getServiceStatus()){
			valuesEqual = true;
		}		
		return valuesEqual;	
	}
	
	public AccessPolicy getPolicy(String userID){
		
		File f= new File("Policy//Access_policy"+userID);
		FileInputStream in;
		try {
			in = new FileInputStream(f);
		ObjectInputStream s = new ObjectInputStream(in);
			policy = (AccessPolicy)s.readObject();
			s.close();
		if(in!=null)in.close();
		} catch (FileNotFoundException e) {
			policy=new AccessPolicy(userID);	
			} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		
		return policy;
	}

	
	//hvordan lese ut samme objekt som klassen man er i???
	public Matrix getAccessMatrix(String userID) {
		
		File f= new File("Policy//Access_Matrix"+userID);
		FileInputStream in;
		try {
			in = new FileInputStream(f);
		ObjectInputStream s = new ObjectInputStream(in);
			matrix = (Matrix)s.readObject();
			s.close();
		if(in!=null)in.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
				
			return matrix;
		}
	
	public void writeToFile(){
		
		try {
			File fil =new File("Policy//"+userID);
			if(fil.exists())fil.delete();
			FileOutputStream out=new FileOutputStream(fil);
			ObjectOutputStream s = new ObjectOutputStream(out);
			s.writeObject(this);
			s.flush();
			s.close();
			if(out!=null)out.close();
					
		} 
		catch (FileNotFoundException e) {
				e.printStackTrace();
		} catch (IOException e) {
				e.printStackTrace();
		}
	}
					
}